/**
   GRAB Graph Layout and Browser System

   Copyright (c) 1989, Tera Computer Company
 **/

// A simple list of objects representing the displayed graph, plus a whole
// lot of things we can do to them

#ifndef graph_h
#define graph_h

#include "att.h"
#include <InterViews/painter.h>
#include <InterViews/brush.h>

class BoxObj;
class MyLabel;
class FontHandler;
class Graphic;

  // an object is either a node, and edge, a dummy node, or a line
enum GDType {node, edge, dummy, linet};	

Brush* brushes[numbrush];
Color* palette[numcolor];

// a representation of a graphical object

class MyGraphic
{
private:
    Coord x0, y0, x1, y1;	// 4 coordinates, which have different meanings
    NShape shape;		// depending on the shape
    Painter* painter;		// degas, I think
    BType brush;		// how to paint it
    CType color;		// color to paint it
public:
    MyGraphic (Coord, Coord, Coord, Coord, NShape s, BType, CType, Painter*);
    ~MyGraphic();
    Painter* GetPainter() { return painter; };
    NShape GetShape() { return shape; };
    BType GetBrush() { return brush; };
    CType GetColor() { return color; };
    void SetPainter(Painter *p) { painter = p; };
    void SetShape(NShape s) { shape = s; };
    void SetBrush(BType b) { brush = b; };
    void SetColor(CType c) { color = c; };
    void GetOriginal (Coord& x, Coord& y, Coord& xx, Coord& yy)
	{ x = x0; y = y0; xx = x1; yy = y1; };
    void Rotate(double, Coord, Coord);
    void Translate(Coord, Coord);
    void Scale(float, float);
    int Width();
    int Height();
    Coord XCenter();
    Coord YCenter();
    Coord YTop();
    Coord YBottom();
};

// the objects in the list

class GraphData 
{
public:
    GraphData ();
    GraphData (GDType, char*);		// used for intersect info 
					// i.e. not really in the list
    GraphData (MyGraphic*, GDType, char*, boolean, char*, int o = -1);
    ~GraphData();
protected:
    MyGraphic* gr;	// the graphical representation
    GDType gdtype;	// is it a node, edge, etc?
    char* gpart;	// pointer to a structure defined in digraph.h
    boolean text;	// should the label be printed
    char* lstr;		// the label
    GraphData* next;
    int ord;		
      /* for multigraph purposes.  Dummy nodes have ords, too! */
private:
    friend class Graph;
    friend class GView;
};

// a node or dummy object

class GNode : public GraphData 
{
public:
    GNode (MyGraphic*, GDType, char*, boolean, char*, char*, char*, int);
private:
    friend class Graph;
    ~GNode();

    Graphic* makeFill(NShape, Coord, Coord, Coord, Coord);

    Graphic* grf;		/* a filled graphic for easy intersect 
				   testing */
    char* upstr;		/* up and down barycenters */
    char* downstr;
};

// an edge or linet object

class GEdge : public GraphData 
{
public:
    GEdge (MyGraphic*, GDType, char*, boolean, char*, boolean, MyGraphic*, 
	MyGraphic*, int);
private:
    friend class Graph;
    ~GEdge();

    boolean arrow;		// does it have arrows
    MyGraphic* arr1;		// arrows, if arrow is true
    MyGraphic* arr2;
};

class Graph 
{
private:
    GraphData* head;		// the list of objects
    FontHandler* fhandler;	// for font info
    GView* gb;			// so we can pick on the guy upstairs
    void Append(GraphData*);
    void MakeArrowHeads (Coord, Coord, Coord, Coord, MyGraphic**, MyGraphic**, 
	BType, CType, Painter*);
    void DelNode (GNode*);
    void DelEdge (GEdge*);
    GNode* RemoveNode(char*, int, boolean any = false);
    GEdge* RemoveEdge(char*, char*, int, boolean any = false);
    GEdge* FindEdge(char*, char*, int);
    GNode* FindNode(char*);
    void ChangeAllLabels(boolean, boolean);
    Painter* ndfault;		// starting point for node painters
    Painter* edfault;		// starting point for edge painters
    void DrawGD(GraphData*);
    void DrawGraphic(GraphData*);
    void DrawLabel(GNode*);
    void DrawBC(GNode*);
    void DrawLabel(GEdge*);
    void DrawArrows(GEdge*);
    void EraseGraphic(GraphData*);
    void EraseLabel(GNode*);
    void EraseBC(GNode*);
    void EraseLabel(GEdge*);
    void EraseArrows(GEdge*);
public:
    Graph();
    ~Graph();
    void AddLine (Coord, Coord, Coord, Coord, BType, CType, boolean);
    void AddEdge (Coord, Coord, Coord, Coord, boolean, char*, char*, BType,
	CType, boolean, int);
    void AddNode (Coord, Coord, Coord, Coord, NShape, char*, boolean, char*,
	char*, char*, BType, CType, boolean, int);
    GraphData* Head () { return head; }
    void DeleteAll();
    void SetFHandler (FontHandler*);
    int myGraphicsIntersecting(BoxObj&, GraphData**&);
    void ChangeMarkDummy();
    void ChangeLabels();
    void ChangePA();
    void ChangeForceNL();
    void ChangeForceEL();
    void ChangeNodeLabel(char*, char*);
    void ChangeEdgeLabel(char*, char*, int, char*);
    void SetGB(GView*);
    void DelAllNodes (char*);
    void DelNode (char*, int);
    void DelAllEdges (char*, char*);
    void DelEdge (char*, char*, int);
    void ChangeEdge (char*, char*, int, BType, CType);
    void ChangeNode (char*, NShape, BType, CType);
    void SetupMove (char*, char*[], char*[], int, int, Coord*, Coord*);
    void ChangeBC();
    void DrawAll();
};

#endif
